@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.46 2004/02/27 22:19:05 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.46.2.3 2004/05/05 09:36:52 fuyu Exp $
  */
 
 #include "config.h"
@@ -60,21 +60,18 @@ static void bbs_thread_view_class_init(BBSThreadViewClass *klass);
 static void bbs_thread_view_init(BBSThreadView *view);
 static void bbs_thread_view_finalize(GObject *object);
 static void bbs_thread_view_destroy(GtkObject *object);
+static void bbs_thread_view_realize(GtkWidget *widget);
 
-static void text_view_realize_cb(GtkTextView *text_view,
-				 BBSThreadView *bbs_thread_view);
-
-static gboolean text_view_get_iter_at_link_tag_from_event(
-						GtkTextView *text_view,
-						GtkTextIter *iter,
-						GdkEventAny *event);
 static gboolean bbs_thread_view_motion_notify(GtkWidget *widget,
 					      GdkEventMotion *event);
+static gboolean bbs_thread_view_leave_notify(GtkWidget *widget,
+					     GdkEventCrossing *event);
 static gboolean bbs_thread_view_button_press(GtkWidget *widget,
 					     GdkEventButton *event);
 static gboolean bbs_thread_view_button_release(GtkWidget *widget,
 					       GdkEventButton *event);
-static void emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event);
+static void emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event,
+				 const gchar *link);
 static void emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event);
 
 static void bbs_thread_view_scroll_to(BBSThreadView *view, int to_where);
@@ -130,7 +127,7 @@ enum {
 
 static GtkTextViewClass *parent_class = NULL;
 static int bbs_thread_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkTextTagTable *default_tag_table;
 
 static GtkTextTag *link_tags[2];
@@ -312,11 +309,10 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(BBSThreadViewClass, link_mouse_out),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__BOXED_POINTER_STRING,
-		 G_TYPE_NONE, 3,
+		 ochusha_marshal_VOID__BOXED_POINTER,
+		 G_TYPE_NONE, 2,
 		 GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE,
-		 G_TYPE_POINTER,
-		 G_TYPE_STRING);
+		 G_TYPE_POINTER);
   bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL] =
     g_signal_new("link_mouse_press",
 		 G_TYPE_FROM_CLASS(klass),
@@ -435,7 +431,9 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
   gtk_binding_entry_add_signal(binding_set, GDK_W, GDK_CONTROL_MASK,
 			       "close_thread", 0);
 
+  widget_class->realize = bbs_thread_view_realize;
   widget_class->motion_notify_event = bbs_thread_view_motion_notify;
+  widget_class->leave_notify_event = bbs_thread_view_leave_notify;
   widget_class->button_press_event = bbs_thread_view_button_press;
   widget_class->button_release_event = bbs_thread_view_button_release;
 
@@ -463,19 +461,15 @@ trace_free(gpointer pointer)
 static void
 bbs_thread_view_init(BBSThreadView *view)
 {
-  GtkWidget *widget = GTK_WIDGET(view);
   GtkTextBuffer *buffer = gtk_text_buffer_new(default_tag_table);
 
-  gtk_text_view_set_buffer(GTK_TEXT_VIEW(view), buffer);
+  gtk_text_view_set_buffer(&view->text_view, buffer);
 
   view->text_buffer = buffer;
 
-  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_CHAR);
-  gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
-  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
-
-  g_signal_connect(G_OBJECT(widget), "realize",
-		   G_CALLBACK(text_view_realize_cb), view);
+  gtk_text_view_set_wrap_mode(&view->text_view, GTK_WRAP_CHAR);
+  gtk_text_view_set_editable(&view->text_view, FALSE);
+  gtk_text_view_set_cursor_visible(&view->text_view, FALSE);
 
   gtk_text_buffer_get_start_iter(buffer, &view->iter);
 
@@ -574,32 +568,38 @@ bbs_thread_view_destroy(GtkObject *object)
 
 
 static void
-text_view_realize_cb(GtkTextView *text_view, BBSThreadView *bbs_thread_view)
+bbs_thread_view_realize(GtkWidget *widget)
 {
-  GdkDisplay *display = gtk_widget_get_display(GTK_WIDGET(text_view));
-  GdkWindow *window = gtk_text_view_get_window(text_view,
-					       GTK_TEXT_WINDOW_TEXT);
+  BBSThreadView *view;
+  GdkDisplay *display;
+  GdkWindow *window;
+
+  if (GTK_WIDGET_CLASS(parent_class)->realize)
+    (*GTK_WIDGET_CLASS(parent_class)->realize)(widget);
+
+  window = gtk_text_view_get_window(GTK_TEXT_VIEW(widget),
+				    GTK_TEXT_WINDOW_TEXT);
   gdk_window_set_events(window,
 			(gdk_window_get_events(window)
 			 & ~GDK_POINTER_MOTION_HINT_MASK)
 			| GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
 
-  if (bbs_thread_view->ibeam_cursor != NULL)
-    gdk_cursor_unref(bbs_thread_view->ibeam_cursor);
-  bbs_thread_view->ibeam_cursor = gdk_cursor_new_for_display(display,
-							     GDK_XTERM);
+  display = gtk_widget_get_display(widget);
+  view = BBS_THREAD_VIEW(widget);
 
-  if (bbs_thread_view->hand_cursor != NULL)
-    gdk_cursor_unref(bbs_thread_view->hand_cursor);
-  bbs_thread_view->hand_cursor = gdk_cursor_new_for_display(display,
-							    GDK_HAND2);
+  if (view->ibeam_cursor != NULL)
+    gdk_cursor_unref(view->ibeam_cursor);
+  view->ibeam_cursor = gdk_cursor_new_for_display(display, GDK_XTERM);
+
+  if (view->hand_cursor != NULL)
+    gdk_cursor_unref(view->hand_cursor);
+  view->hand_cursor = gdk_cursor_new_for_display(display, GDK_HAND2);
 }
 
 
 static gboolean
-text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
-					  GtkTextIter *iter,
-					  GdkEventAny *event)
+text_view_get_iter_from_event(GtkTextView *text_view, GtkTextIter *iter,
+			      GdkEventAny *event)
 {
   int buf_x;
   int buf_y;
@@ -630,7 +630,13 @@ text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
     }
 
   gtk_text_view_get_iter_at_location(text_view, iter, buf_x, buf_y);
+  return TRUE;
+}
 
+
+static gboolean
+text_view_move_iter_at_link_tag(GtkTextView *text_view, GtkTextIter *iter)
+{
   if (gtk_text_iter_has_tag(iter, link_tags[0]))
     {	/* link_tags[0]ǰϤޤƤϰϤؤƤ */
       if (gtk_text_iter_begins_tag(iter, link_tags[0]))
@@ -684,63 +690,10 @@ text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
 }
 
 
-static gboolean
-bbs_thread_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
-{
-  gboolean result = FALSE;
-  GtkTextIter iter;
-  GtkTextView *text_view;
-  BBSThreadView *bbs_thread_view;
-
-  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
-		       && event->type == GDK_MOTION_NOTIFY, FALSE);
-
-  text_view = GTK_TEXT_VIEW(widget);
-  bbs_thread_view = BBS_THREAD_VIEW(widget);
-
-  if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
-						(GdkEventAny *)event))
-    {
-      int start_offset = gtk_text_iter_get_offset(&iter);
-      if (start_offset == bbs_thread_view->last_offset)
-	{
-	  goto finished;
-	}
-
-      if (bbs_thread_view->last_offset != -1)
-	emit_link_mouse_out(bbs_thread_view, (GdkEventAny *)event);
-
-      bbs_thread_view->last_offset = start_offset;
-      emit_link_mouse_over(bbs_thread_view, event);
-    }
-  else if (bbs_thread_view->last_offset != -1)
-    emit_link_mouse_out(bbs_thread_view, (GdkEventAny *)event);
-
- finished:
-  if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event != NULL)
-    result = (*GTK_WIDGET_CLASS(parent_class)->motion_notify_event)(widget,
-								    event);
-
-  return result;
-}
-
-
 static void
-emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
+emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event,
+		     const gchar *link)
 {
-  gchar *link;
-  if (view->last_offset == -1)
-    return;
-
-  if (view->link_table == NULL)
-    return;
-
-  link = (gchar *)g_hash_table_lookup(view->link_table,
-				      (gconstpointer)view->last_offset);
-
-  if (link == NULL)
-    return;
-
   gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(view),
 						 GTK_TEXT_WINDOW_TEXT),
 			view->hand_cursor);
@@ -757,21 +710,6 @@ emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
 static void
 emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 {
-  gchar *link;
-  if (view->last_offset == -1)
-    return;
-
-  if (view->link_table == NULL)
-    return;
-
-  link = (gchar *)g_hash_table_lookup(view->link_table,
-				      (gconstpointer)view->last_offset);
-
-  view->last_offset = -1;
-
-  if (link == NULL)
-    return;
-
   gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(view),
 						 GTK_TEXT_WINDOW_TEXT),
 			view->ibeam_cursor);
@@ -780,47 +718,140 @@ emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 		bbs_thread_view_signals[LINK_MOUSE_OUT_SIGNAL],
 		0,
 		event,
-		view->thread,
-		link);
+		view->thread);
+}
+
+
+static gboolean
+bbs_thread_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
+{
+  GtkTextIter iter;
+  GtkTextView *text_view;
+  BBSThreadView *view;
+
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
+		       && event->type == GDK_MOTION_NOTIFY, FALSE);
+
+  view = BBS_THREAD_VIEW(widget);
+  text_view = &view->text_view;
+
+  if (text_view_get_iter_from_event(text_view, &iter, (GdkEventAny *)event))
+    {
+      GtkTextIter start;
+      GtkTextIter end;
+      gchar *text;
+
+      if (gtk_text_buffer_get_selection_bounds(view->text_buffer, &start, &end)
+	  && gtk_text_iter_in_range(&iter, &start, &end)
+	  && (text = gtk_text_iter_get_visible_text(&start, &end)) != NULL)
+	{
+	  int start_offset = gtk_text_iter_get_offset(&iter);
+	  if (start_offset != view->last_offset)
+	    {
+	      if (view->last_offset != -1)
+		emit_link_mouse_out(view, (GdkEventAny *)event);
+
+	      emit_link_mouse_over(view, event, text);
+	      view->last_offset = start_offset;
+	    }
+
+	  g_free(text);
+	}
+      else if (text_view_move_iter_at_link_tag(text_view, &iter))
+	{
+	  int start_offset = gtk_text_iter_get_offset(&iter);
+	  if (start_offset != view->last_offset)
+	    {
+	      const gchar *link;
+
+	      if (view->last_offset != -1)
+		emit_link_mouse_out(view, (GdkEventAny *)event);
+
+	      if (view->link_table != NULL
+		  && (link = g_hash_table_lookup(view->link_table,
+				(gconstpointer)start_offset)) != NULL)
+		{
+		  emit_link_mouse_over(view, event, link);
+		  view->last_offset = start_offset;
+		}
+	      else
+		view->last_offset = -1;
+	    }
+	}
+      else if (view->last_offset != -1)
+	{
+	  emit_link_mouse_out(view, (GdkEventAny *)event);
+	  view->last_offset = -1;
+	}
+    }
+  else if (view->last_offset != -1)
+    {
+      emit_link_mouse_out(view, (GdkEventAny *)event);
+      view->last_offset = -1;
+    }
+
+  if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event != NULL)
+    return (*GTK_WIDGET_CLASS(parent_class)->motion_notify_event)(widget,
+								  event);
+  return FALSE;
+}
+
+
+static gboolean
+bbs_thread_view_leave_notify(GtkWidget *widget, GdkEventCrossing *event)
+{
+  BBSThreadView *view = BBS_THREAD_VIEW(widget);
+
+  if (view->last_offset != -1)
+    {
+      emit_link_mouse_out(view, (GdkEventAny *)event);
+      view->last_offset = -1;
+    }
+
+  if (GTK_WIDGET_CLASS(parent_class)->leave_notify_event != NULL)
+    return (*GTK_WIDGET_CLASS(parent_class)->leave_notify_event)(widget,
+								 event);
+  return FALSE;
 }
 
 
 static gboolean
 bbs_thread_view_button_press(GtkWidget *widget, GdkEventButton *event)
 {
+  gboolean result = FALSE;
+  BBSThreadView *view;
   GtkTextView *text_view;
   GtkTextIter iter;
-  gboolean result = FALSE;
 
   g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
 		       && (event->type == GDK_BUTTON_PRESS
 			   || event->type == GDK_2BUTTON_PRESS
 			   || event->type == GDK_3BUTTON_PRESS), FALSE);
-  text_view = GTK_TEXT_VIEW(widget);
+  view = BBS_THREAD_VIEW(widget);
+  text_view = &view->text_view;
 
-  if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
-						(GdkEventAny *)event))
+  if (text_view_get_iter_from_event(text_view, &iter, (GdkEventAny *)event)
+      && text_view_move_iter_at_link_tag(text_view, &iter))
     {
-      BBSThreadView *bbs_thread_view = BBS_THREAD_VIEW(text_view);
-      int start_offset = gtk_text_iter_get_offset(&iter);
-      gchar *link = (bbs_thread_view->link_table != NULL)
-	? (gchar *)g_hash_table_lookup(bbs_thread_view->link_table,
-				       (gconstpointer)start_offset)
-	: NULL;
-
-      g_signal_emit(G_OBJECT(bbs_thread_view),
-		    bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL],
-		    0,
-		    event,
-		    bbs_thread_view->thread,
-		    link,
-		    &result);
+      int offset = gtk_text_iter_get_offset(&iter);
+      const gchar *link
+	= (view->link_table != NULL
+	   ? (gchar *)g_hash_table_lookup(view->link_table,
+					  (gconstpointer)offset)
+	   : NULL);
+      if (link != NULL)
+	g_signal_emit(G_OBJECT(view),
+		      bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL],
+		      0,
+		      event,
+		      view->thread,
+		      link,
+		      &result);
     }
 
   if (!result && GTK_WIDGET_CLASS(parent_class)->button_press_event != NULL)
-    result = (*GTK_WIDGET_CLASS(parent_class)->button_press_event)(widget,
-								   event);
-
+    return (*GTK_WIDGET_CLASS(parent_class)->button_press_event)(widget,
+								 event);
   return result;
 }
 
@@ -828,37 +859,39 @@ bbs_thread_view_button_press(GtkWidget *widget, GdkEventButton *event)
 static gboolean
 bbs_thread_view_button_release(GtkWidget *widget, GdkEventButton *event)
 {
+  gboolean result = FALSE;
+  BBSThreadView *view;
   GtkTextView *text_view;
   GtkTextIter iter;
-  gboolean result = FALSE;
 
-  g_return_val_if_fail(GTK_IS_TEXT_VIEW(widget)
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
 		       && event->type == GDK_BUTTON_RELEASE, FALSE);
-  text_view = GTK_TEXT_VIEW(widget);
+  view = BBS_THREAD_VIEW(widget);
+  text_view = &view->text_view;
 
-  if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
-						(GdkEventAny *)event))
+  if (text_view_get_iter_from_event(text_view, &iter, (GdkEventAny *)event)
+      && text_view_move_iter_at_link_tag(text_view, &iter))
     {
-      BBSThreadView *bbs_thread_view = BBS_THREAD_VIEW(text_view);
-      int start_offset = gtk_text_iter_get_offset(&iter);
-      gchar *link = (bbs_thread_view->link_table != NULL)
-	? (gchar *)g_hash_table_lookup(bbs_thread_view->link_table,
-				       (gconstpointer)start_offset)
-	: NULL;
+      int offset = gtk_text_iter_get_offset(&iter);
+      const gchar *link
+	= (view->link_table != NULL
+	   ? (gchar *)g_hash_table_lookup(view->link_table,
+					  (gconstpointer)offset)
+	   : NULL);
 
-      g_signal_emit(G_OBJECT(bbs_thread_view),
-		    bbs_thread_view_signals[LINK_MOUSE_RELEASE_SIGNAL],
-		    0,
-		    event,
-		    bbs_thread_view->thread,
-		    link,
-		    &result);
+      if (link != NULL)
+	g_signal_emit(G_OBJECT(view),
+		      bbs_thread_view_signals[LINK_MOUSE_RELEASE_SIGNAL],
+		      0,
+		      event,
+		      view->thread,
+		      link,
+		      &result);
     }
 
   if (GTK_WIDGET_CLASS(parent_class)->button_release_event != NULL)
-    result = (*GTK_WIDGET_CLASS(parent_class)->button_release_event)(widget,
-								     event);
-
+    return (*GTK_WIDGET_CLASS(parent_class)->button_release_event)(widget,
+								   event);
   return result;
 }
 
@@ -891,6 +924,20 @@ bbs_thread_view_get_thread(BBSThreadView *view)
 }
 
 
+char *
+bbs_thread_view_get_selected_text(BBSThreadView *view)
+{
+  GtkTextIter start;
+  GtkTextIter end;
+
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view), NULL);
+
+  if (!gtk_text_buffer_get_selection_bounds(view->text_buffer, &start, &end))
+    return NULL;	/* 򤵤Ƥʤ */
+  return gtk_text_iter_get_visible_text(&start, &end);
+}
+
+
 void
 bbs_thread_view_clear(BBSThreadView *view)
 {
