@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.92.2.1 2004/05/01 04:34:27 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.92.2.4 2004/05/24 10:09:35 fuyu Exp $
  */
 
 #include "config.h"
@@ -144,6 +144,10 @@ static void item_view_closed_cb(PanedNotebook *paned_notebook,
 static void page_switched_cb(PanedNotebook *paned_notebook,
 			     GtkWidget *previous_page, GtkWidget *new_page,
 			     OchushaApplication *application);
+static void page_double_clicked_cb(PanedNotebook *paned_notebook,
+				   OchushaBulletinBoard *board,
+				   GtkWidget *item_view,
+				   OchushaApplication *application);
 
 static void make_new_thread_button_cb(GObject *unused,
 				      OchushaApplication *application);
@@ -540,6 +544,9 @@ initialize_boardlist(OchushaApplication *application)
   g_signal_connect(G_OBJECT(application->contents_window),
 		   "page_switched", G_CALLBACK(page_switched_cb),
 		   application);
+  g_signal_connect(G_OBJECT(application->contents_window),
+		   "page_double_clicked", G_CALLBACK(page_double_clicked_cb),
+		   application);
 
   application->threadlist_search_window = NULL;
 
@@ -1499,6 +1506,12 @@ item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBulletinBoard *board,
   gtk_widget_hide(item_view);
   gtk_widget_unrealize(item_view);
   delayed_g_object_unref(G_OBJECT(item_view));
+
+  if (ochusha_bulletin_board_trylock_thread_list(board))
+    {
+      write_threadlist(board, application);
+      ochusha_bulletin_board_unlock_thread_list(board);
+    }
 }
 
 
@@ -1563,6 +1576,15 @@ page_switched_cb(PanedNotebook *paned_notebook,
 }
 
 
+static void
+page_double_clicked_cb(PanedNotebook *paned_notebook,
+		       OchushaBulletinBoard *board, GtkWidget *item_view,
+		       OchushaApplication *application)
+{
+  refresh_threadlist_view(application, board);
+}
+
+
 static OchushaBulletinBoard *
 board_new_cb(OchushaBBSTable *table, int bbs_type, const gchar *name,
 	     const char *base_url, OchushaApplication *application)
@@ -2001,18 +2023,12 @@ process_board_move(OchushaBulletinBoard *old_board,
 		   OchushaBulletinBoard *new_board,
 		   gpointer user_data)
 {
-#if 0
   OchushaApplication *application = (OchushaApplication *)user_data;
-#endif  
-  /*
-   * TODO: DATեʤɥå夷ƤǡΰžԤ٤
-   *       İžɤߤGUIɲäɬפ⡣
-   */
-#if DEBUG_GUI
-  fprintf(stderr, "process_board_move: old_board=%p new_board=%p\n",
-	  old_board, new_board);
-#endif
-  return new_board;	/* ǤϿΤͥ */
+  const char *new_base_url = ochusha_bulletin_board_get_base_url(new_board);
+  ochusha_bulletin_board_process_board_move(old_board, new_base_url,
+					    &application->config);
+
+  return old_board;
 }
 
 
